//  FILE:       Execute.h 
//              Microsoft TPC-H Kit Ver. 1.00
//              Copyright Microsoft, 1999
//              All Rights Reserved
//
//
//  PURPOSE:    Declaration of the CExecute
//  Contact:    Reshma Tharamal (reshmat@microsoft.com)
// 
// Execute.h : Declaration of the CExecute

#ifndef __EXECUTE_H_
#define __EXECUTE_H_

#include <atlwin.h>
#include <comdef.h>
#include <stdio.h>
#include "resource.h"       // main symbols
#include "ExecuteDllCP.h"
#include "..\LogWriter\LogWriter.h"
#include "..\LogWriter\SMLog.h"
#include "..\common\SMTime\SMTime.h"
#include "..\common\SMTime\SMTimer.h"

// ODBC-specific includes 
#define DBNTWIN32
#include <sqltypes.h>
#include <sql.h>
#include <sqlext.h>

/////////////////////////////////////////////////////////////////////////////
// CExecute
      
#define WM_TASK_START  (WM_USER + 101)
#define WM_TASK_FINISH  (WM_USER + 102)

#define SM_SQLERR_FORMAT		"SQL Error State:%s, Native Error Code: %ld\r\nODBC Error: %s"
													// format for ODBC error messages
#define SM_SQLWARN_FORMAT		SM_SQLERR_FORMAT	// format for ODBC warnings 
#define SM_SQLMSG_FORMAT		"%s"				// format for messages from the server

#define SM_SQL_STATE_WARNING	"01000"
#define SM_MSG_SERVER			"[Microsoft][ODBC SQL Server Driver][SQL Server]"

#define SM_SQL_ERR_CHANGED_DB	5701
#define SM_SQL_ERR_CHANGED_LANG	5703

#define SM_STEPMASTER_ERROR		"StepMaster Error: "

#define CMD_SEPARATOR			"\nGO"

#define INV_ARRAY_INDEX     -1			// invalid index into an array

#define MAXBUFLEN           256         // display buffer size
#define MAXLOGCMDLEN        256         // maximum characters in command that will be 
										// printed to log
#define MAXLOGCMDBUF        512         // maximum characters in command that will be 
										// printed to log
#define MAX_DATA_LEN        4000         // maximum buffer size for variable-length data types 
										// viz. character and binary fields
#define FILE_ACCESS_READ    "r"			// Open file for read access

#define S_NULL	"NULL"

// Define a macro to increase the size of a buffer so it is a multiple of teh alignment size.
// Thus, if a buffer starts on an alignment boundary, it will end just before the next 
// alignment boundary. Here, an alignment size of 4 is used because this is the size of the 
// largest data type used in the application's buffer - the size of an SDWORD and of the largest
// default C data type are both 4. If a larger data type (such as __int64) is used, it will be
// necessary to align for that size.
#define ALIGNSIZE 4
#define ALIGNBUF(Length) ((Length) % ALIGNSIZE) ? \
    ((Length) + ALIGNSIZE - ((Length) % ALIGNSIZE)) : (Length)

#define MAX_BUFFER_SIZE		64000

typedef enum OdbcOperations 
{
	SMSQLAllocHandle,
	SMSQLDriverConnect,
	SMSQLExecDirect,
	SMSQLSetStmtAttr,
	SMSQLCancel,
	SMSQLNumResultCols,
	SMSQLDescribeCol,
	SMSQLColAttribute,
	SMSQLFetch,
	SMSQLGetData,
	SMSQLRowCount,
	SMSQLMoreResults,
	SMSQLBindCol,
};

class CODBCError
{
public:
	CODBCError(SQLRETURN nResult, SWORD fHandleType, SQLHANDLE	handle, OdbcOperations FailedOp)
	{
		m_fHandleType	= fHandleType;
		m_handle		= handle;
		m_FailedOp		= FailedOp;
		m_nResult		= nResult;
	};

private:
	SWORD			m_fHandleType;
	SQLHANDLE		m_handle;
	OdbcOperations	m_FailedOp;
	SQLRETURN		m_nResult;

private:
inline BOOL IsServerMessage(SQLINTEGER lNativeError, UCHAR *szErr){
	return( (strstr((LPCTSTR)szErr, SM_MSG_SERVER) != NULL) ? (lNativeError == 0) : FALSE); }
inline BOOL IsODBCWarning(UCHAR *szSqlState){	return(strcmp((LPCSTR)szSqlState, SM_SQL_STATE_WARNING) == 0);}
inline LPCSTR ParseOdbcMsgPrefixes(LPCSTR szMsg){ char *pDest;
	return( (pDest = strstr(szMsg, SM_MSG_SERVER)) == NULL ? szMsg : pDest + strlen(SM_MSG_SERVER));}
};

class ATL_NO_VTABLE CExecute : 
	public CWindowImpl<CExecute>,
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CExecute, &CLSID_Execute>,
	public IConnectionPointContainerImpl<CExecute>,
	public ISupportErrorInfo,
	public IDispatchImpl<IExecute, &IID_IExecute, &LIBID_EXECUTEDLLLib>,
	public CProxy_IExecuteEvents< CExecute >
{
public:
	CExecute() 
	{
		m_pErrorFile = NULL;
		//m_pLogFile = NULL;
		m_pOutputFile = NULL;

		// Initialize the elapsed time for the step
		m_tElapsedTime = 0;

		// Initialize the run status for the step
		m_StepStatus = gintPending;

		m_hHandle = SQL_NULL_HSTMT;
		m_bAbort = FALSE;

		m_iConnectionIndex = INV_ARRAY_INDEX;
	}

	~CExecute()
	{
	}

	friend class CExecError;

public:
	DECLARE_WND_CLASS("Execute")
		
		BEGIN_MSG_MAP(CExecute)
		MESSAGE_HANDLER(WM_TASK_FINISH, OnTaskFinished)
		MESSAGE_HANDLER(WM_TASK_START, OnTaskStarted)
		END_MSG_MAP()
public:

	LRESULT OnTaskStarted(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		CURRENCY	CStartTime = Get64BitTime(&m_tStartTime);

		Fire_Start(CStartTime);
		return 0;
	}
	
	LRESULT OnTaskFinished(UINT uMsg, WPARAM wParam,
		LPARAM lParam, BOOL& bHandled)
	{
		CURRENCY	CEndTime = Get64BitTime(&m_tEndTime);

		Fire_Complete(CEndTime, (long)m_tElapsedTime);
		return 0;
	}
	
	HRESULT FinalConstruct()
	{
		HRESULT		hr;
		RECT		rect;

		rect.left=0;
		rect.right=100;
		rect.top=0;
		rect.bottom=100;

		HWND hwnd = Create( NULL, rect, "ExecuteWindow", WS_POPUP);
		
		if (!hwnd)
			return HRESULT_FROM_WIN32(GetLastError());

		hr = CoCreateInstance(CLSID_SMLog, NULL, CLSCTX_INPROC, 
			IID_ISMLog, (void **)&m_pErrorFile);
		if FAILED(hr) 
			return(hr);
		m_pErrorFile->put_Append(TRUE);

		//hr = CoCreateInstance(CLSID_SMLog, NULL, CLSCTX_INPROC, 
		//	IID_ISMLog, (void **)&m_pLogFile);
		//if FAILED(hr) 
		//	return(hr);

		hr = CoCreateInstance(CLSID_SMLog, NULL, CLSCTX_INPROC, 
			IID_ISMLog, (void **)&m_pOutputFile);
		if FAILED(hr) 
			return(hr);
		m_pOutputFile->put_Append(TRUE);

		hr = CoCreateInstance(CLSID_SMTimer, NULL, CLSCTX_INPROC, 
			IID_ISMTimer, (void **)&m_ExecTime);
		if FAILED(hr) 
			return(hr);
		
		return S_OK;
	}
	
	void FinalRelease()
	{
		if (m_hWnd != NULL)
			DestroyWindow();

		// Close the log and error files
		if (m_pErrorFile)
			m_pErrorFile->Release();
		m_pErrorFile = NULL;
		
		//if (m_pLogFile)
		//	m_pLogFile->Release();
		//m_pLogFile = NULL;
		
		if (m_ExecTime)
			m_ExecTime->Release();
		m_ExecTime = NULL;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_EXECUTE)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CExecute)
	COM_INTERFACE_ENTRY(IExecute)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(CExecute)
CONNECTION_POINT_ENTRY(DIID__IExecuteEvents)
END_CONNECTION_POINT_MAP()

// ISupportsErrorInfo
STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IExecute
public:
	STDMETHOD(put_ErrorFile)(/*[in]*/ BSTR newVal);
	STDMETHOD(put_OutputFile)(/*[in]*/ BSTR newVal);
	STDMETHOD(WriteError)(BSTR szMsg);
	STDMETHOD(Abort)();
	STDMETHOD(get_StepStatus)(/*[out, retval]*/ InstanceStatus *pVal);
	STDMETHOD(DoExecute)(/*[in]*/ BSTR szCommand, /*[in]*/ BSTR szExecutionDtls, /*[in]*/ ExecutionType ExecMethod,
		/*[in]*/ BOOL bNoCount, /*[in]*/ BOOL bNoExecute, /*[in]*/ BOOL bParseOnly, 
		/*[in]*/ BOOL bQuotedIds, /*[in]*/ BOOL bAnsiNulls, /*[in]*/ BOOL bShowQP, 
		/*[in]*/ BOOL bStatsTime, /*[in]*/ BOOL bStatsIO, /*[in]*/ long lRowCount, 
		/*[in]*/ long lQueryTmout, /*[in]*/ BSTR szConnection);

	TC_TIME			ExecuteShell();
	TC_TIME			ExecuteODBC();
	STDMETHODIMP	AbortShell();
	STDMETHODIMP	AbortODBC();

	_bstr_t			m_szCommand;
	_bstr_t			m_szExecDtls;
	_bstr_t			m_szConnection;
	DWORD			m_lMode;
	//DATE			m_CurTime;
	SYSTEMTIME		m_tStartTime;
	SYSTEMTIME		m_tEndTime;
	TC_TIME			m_tElapsedTime;
	ISMLog			*m_pErrorFile;
	//ILog			*m_pLogFile;
	ISMLog			*m_pOutputFile;
	ISMTimer		*m_ExecTime;
	ExecutionType	m_ExecMthd;
	InstanceStatus	m_StepStatus;
	HANDLE			m_hHandle;		// Process handle for shell commands and 
									// Statement handle for ODBC commands
	LPSTR			m_szCmd;

private:
	LPSTR			NextCmdInBatch(LPSTR szBatch);
	void			ProcessResultsets();
	SQLSMALLINT		GetDefaultCType(SQLINTEGER SQLType);
	void			PrintData(void *vData, SQLSMALLINT CType, SQLINTEGER IndPtr, SQLSMALLINT iScale, ISMLog *pOutput);
	void			LogODBCErrors(SQLRETURN nResult, SWORD fHandleType, SQLHANDLE handle, OdbcOperations FailedOp);
	void			ODBCCleanup(HDBC *hdbc, HSTMT *hstmt);
	STDMETHODIMP	RaiseSystemError(void);
	void			LogSystemError(ISMLog *pFile);
	void			GetSystemError(LPSTR s);
	void			SetConnectionOption(LPSTR szConn, HDBC *pHdbc);
	void			ResetConnectionProperties(HDBC *p_hdbc);

	void			InitializeConnection(HDBC *phdbc, BOOL *pbDoConnect);
	void			ReConnectDeadConnection(HDBC *phdbc);

	void			ResetConnectionUsage();

	int				m_iConnectionIndex;

	BOOL			m_bNoCount, m_bNoExecute, m_bParseOnly, m_bQuotedIds, m_bAnsiNulls, \
					m_bShowQP, m_bStatsTime, m_bStatsIO;
	long			m_lRowCount;
	SQLUINTEGER		m_lQueryTmout;
	_bstr_t			m_ErrorFile, m_OutputFile;
	BOOL			m_bAbort;

private:
inline BOOL IsServerMessage(SQLINTEGER lNativeError, UCHAR *szErr){
	return( (strstr((LPCTSTR)szErr, SM_MSG_SERVER) != NULL) ? (lNativeError == 0) : FALSE); }
inline BOOL IsODBCWarning(UCHAR *szSqlState){	return(strcmp((LPCSTR)szSqlState, SM_SQL_STATE_WARNING) == 0);}
inline BOOL IsErrorReturn(SQLRETURN iRetCode){	
	return( (!SQL_SUCCEEDED(iRetCode)) && (iRetCode != SQL_NO_DATA) );}
inline LPCSTR ParseOdbcMsgPrefixes(LPCSTR szMsg){ char *pDest;
	return( (pDest = strstr(szMsg, SM_MSG_SERVER)) == NULL ? szMsg : pDest + strlen(SM_MSG_SERVER));}
inline BOOL IsDynamicConnection(){ return(!strcmp((LPSTR)m_szConnection, ""));}
inline void HandleODBCError(SQLRETURN rc, SWORD fHandleType, SQLHANDLE handle, OdbcOperations OdbcOp)
{
	if (rc != SQL_SUCCESS)
    {
		LogODBCErrors(rc, fHandleType, handle, OdbcOp);
		if (IsErrorReturn(rc))
			throw new CODBCError(rc, fHandleType, handle, OdbcOp);
    }
	return;
}

};

class CExecError
{
public:
	typedef enum ExecErrorCodes
	{
		SM_ERR_CONN_IN_USE,
	};
	
	CExecError(int iError)
	{
		m_iErrCode	= iError;
	};
	
	void			LogErrors(CExecute *p);

private:
	int				m_iErrCode;
	static char		*m_szExecErrorDesc[];
};

void			ExecutionThread(LPVOID lpParameter);

#ifdef _TPCH_AUDIT
	void			WriteFileToTpchLog(LPSTR szFile, LPSTR szFmt);
	void			WriteToTpchLog(char *szMsg);
#endif

#endif //__EXECUTE_H_
