Attribute VB_Name = "Iterator"
'  FILE:       Iterator.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality to add, modify, delete
'              and edit iterators. Also other functions
'              specific to connection strings, viz. iterator
'              validation, load and so on.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "Iterator."

Private Const mintFrom As Integer = 1
Private Const mintTo As Integer = 2
Private Const mintStep As Integer = 3

Private Const mintItTypeNumeric As Integer = 1
Private Const mintItTypeValues As Integer = 2

Private Const mintColValue As Integer = 0
Public Const mintMinIterator As Integer = 1

Private Sub ValidateIteratorName(strCheck As String, cStepRec As cStep)
    ' Raises an error if the passed in string occurs as a
    ' parameter or an iterator in the workspace
    Dim cDummyParam As cParameter
    Dim strStepKey As String
    Dim strParentKey As String
    Dim strRootKey As String
    Dim cParentStep As cStep
    
    On Error GoTo ValidateIteratorNameErr
    
    ' Create a dummy parameter record, so we can use an existing
    ' procedure to do the validation
    Set cDummyParam = New cParameter
    Set cDummyParam.NodeDB = dbsAttTool
    cDummyParam.ParameterName = strCheck
    cDummyParam.WorkspaceId = cStepRec.WorkspaceId
    cDummyParam.IndOperation = QueryOp
    gcParameters.Validate cDummyParam
    Set cDummyParam = Nothing
    
    ' Check if the iterator name exists in any of the parents of
    ' this step.
    strStepKey = MakeKeyValid(cStepRec.StepId, cStepRec.StepType)
    strRootKey = LabelStep(cStepRec.WorkspaceId)
                
    Do
        If frmMain.treStepView.Nodes(strStepKey).Parent Is Nothing Then
            strParentKey = gstrEmptyString
        Else
            strParentKey = frmMain.treStepView.Nodes(strStepKey).Parent.Key
            If strParentKey <> strRootKey Then
                Set cParentStep = gcSteps.QueryStep( _
                        MakeIdentifierValid(strParentKey))
                If Not StringEmpty(cParentStep.IteratorName) And _
                        cParentStep.IteratorName = strCheck Then
                    ShowError errDupIterator
                    On Error GoTo 0
                    gstrSource = mstrModuleName & "ValidateIteratorName"
                    Err.Raise vbObjectError + errDupIterator, _
                            gstrSource, LoadResString(errDupIterator)
                End If
            End If
        End If
        
        strStepKey = strParentKey
    Loop Until strParentKey = strRootKey
    
    Exit Sub
    
ValidateIteratorNameErr:
    ' Display an error message if it is due to a duplicate parameter name
    If Err.Number - vbObjectError = errDuplicateParameterName Then
        ShowError errDuplicateParameterName
    End If
    
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "ValidateIteratorName"
    Err.Raise vbObjectError + errValidateFailed, gstrSource, LoadResString(errValidateFailed)
    
End Sub
Public Sub EnteringItName()
    
    On Error GoTo EnteringItNameErr

    If StringEmpty(frmSteps.txtIterator) Then
        ' The user has deleted a previously entered iterator,
        ' if any. Disable the frames to enter both range
        ' and discrete values
        Call SetEnabledForCtlArray(frmSteps.optItType, False)
        
        Call SetRangeEntryEnabled(False)
        Call SetListEntryEnabled(False)
        
    Else
        ' Enable the options buttons that allow the user
        ' to choose whether to enter a range or a list of values
        Call SetEnabledForCtlArray(frmSteps.optItType, True)
        
    End If
    Exit Sub
    
EnteringItNameErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub
Public Sub InitializeIterators()

    On Error GoTo InitializeIteratorsErr
    
    ' Call a procedure to adjust the column width in the grid
    Call InitItValuesGrid(frmSteps.msItValues)
    
    Call EnteringItName
    
    Call InitCtlArray(frmSteps.txtItNumValues, mintMinIterator)
    
    Call SetRangeEntryEnabled(False)
    Call SetListEntryEnabled(False)
    
    Exit Sub

InitializeIteratorsErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub

Public Sub SetRangeEntryEnabled(blnEnabled As Boolean)
    
    On Error GoTo SetRangeEntryEnabledErr
        
    frmSteps.fraItNumValues.Enabled = blnEnabled
    Call SetEnabledForCtlArray(frmSteps.txtItNumValues, _
            blnEnabled)
'    Call SetEnabledForCtlArray(frmSteps.updItNumValues, _
'            blnEnabled)
        
    Exit Sub
    
SetRangeEntryEnabledErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub
Public Sub SetListEntryEnabled(blnEnabled As Boolean)
    
    On Error GoTo SetListEntryEnabledErr
        
    frmSteps.fraItList.Enabled = blnEnabled
    frmSteps.msItValues.Enabled = blnEnabled
        
    Exit Sub
    
SetListEntryEnabledErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub

Public Sub IteratorTypeSelected(intIndex As Integer)
    
    On Error GoTo IteratorTypeSelectedErr

    If intIndex = mintItTypeNumeric Then
        ' Disable the grid to enter a set of values
        Call SetRangeEntryEnabled(True)
        Call SetListEntryEnabled(False)
    Else
        ' Disable the grid to enter a range of values
        Call SetRangeEntryEnabled(False)
        Call SetListEntryEnabled(True)
    End If
    
    Exit Sub
    
IteratorTypeSelectedErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub

Private Sub InitItValuesGrid(msValues As MSFlexGrid)
    
    On Error GoTo InitItValuesGridErr
    
    msValues.ColWidth(mintColValue) = msValues.Width
    msValues.ColAlignment(mintColValue) = flexAlignLeftCenter
    
    Exit Sub

InitItValuesGridErr:
    Call LogErrors(Errors)
    Resume Next
    
End Sub

Private Sub SetValue(msValues As MSFlexGrid, _
        cItRec As cIterator)
        
    On Error GoTo SetValueErr
    
    If cItRec.SequenceNo >= msValues.Rows Then
        msValues.Rows = cItRec.SequenceNo + 1
    End If
    msValues.TextMatrix(cItRec.SequenceNo, _
            mintColValue) = cItRec.Value

    Exit Sub

SetValueErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadDataFailed, _
            mstrModuleName & "SetValue", LoadResString(errLoadDataFailed)

End Sub

Private Function GetIterator(cColIterators As cNodeCollections, _
    intItType As ValueType, _
    strValue As String) As cIterator
    
    ' Returns the matching iterator record if it exists
    ' within the passed in array, else returns nothing
    
    Dim lngIndex As Long

    On Error GoTo GetIteratorErr
    gstrSource = mstrModuleName & "GetIterator"

    If intItType = gintValue Then
        ' Check whether the value entered on the grid has been
        ' already defined for the iterator
        For lngIndex = 0 To cColIterators.Count - 1
            If cColIterators(lngIndex).Value = strValue Then
                Set GetIterator = cColIterators(lngIndex)
                Exit For
            End If
        Next lngIndex
    Else
        ' The iterator is of type numeric - the type is enough
        ' to qualify the iterator record
        For lngIndex = 0 To cColIterators.Count - 1
            If cColIterators(lngIndex).IteratorType = intItType Then
                Set GetIterator = cColIterators(lngIndex)
                Exit For
            End If
        Next lngIndex
    End If

    Exit Function

GetIteratorErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errQueryFailed, gstrSource, _
        LoadResString(errQueryFailed)

End Function
Private Function NewIterators(cColIterators As cNodeCollections, _
    cStepRec As cStep) As cIterator
    
    ' Creates iterator records for all the iterators in the
    ' passed in collection
    Dim cNewIterator As cIterator
    Dim cTemp As cIterator
    Dim lngIndex As Long

    On Error GoTo NewIteratorsErr

    For lngIndex = 0 To cColIterators.Count - 1
        Set cTemp = cColIterators(lngIndex)
        
        Set cNewIterator = New cIterator
        cNewIterator.IteratorType = cTemp.IteratorType
        cNewIterator.SequenceNo = cTemp.SequenceNo
        cNewIterator.Value = cTemp.Value
        
        ' Add the iterator record to the step
        cStepRec.AddIterator cNewIterator
    Next lngIndex

    Exit Function

NewIteratorsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "NewIterators"
    On Error GoTo 0
    Err.Raise vbObjectError + errInsertIteratorFailed, _
            gstrSource, LoadResString(errInsertIteratorFailed)

End Function
Private Function GetItValuesFromScreen() As cNodeCollections
    ' Returns a collection of all the iterator values
    ' entered by the user on the Step Properties screen

    Dim cItValues As cNodeCollections
    Dim cItRecord As cIterator
    Dim intIndex As Integer
    Dim intSequence As Long
    Dim strValue As String
    
    On Error GoTo GetItValuesFromScreenErr
    gstrSource = mstrModuleName & "GetItValuesFromScreen"

    Set cItValues = New cNodeCollections
    
    If Not StringEmpty(frmSteps.txtIterator) Then
        If frmSteps.optItType(mintItTypeNumeric) Then
            ' The user has entered a range of values for the
            ' iterator
            ' Create iterator records of type From, To and Step
            ' Note: These types correspond to the indices in
            ' the control array
            For intIndex = frmSteps.txtItNumValues.LBound To _
                    frmSteps.txtItNumValues.UBound
    
                Set cItRecord = New cIterator
                cItRecord.IteratorType = intIndex
                cItRecord.SequenceNo = 0
                cItRecord.Value = Trim$(frmSteps.txtItNumValues(intIndex).Text)
    
                cItValues.Load cItRecord
            Next intIndex
    
        Else
            ' The user has entered a set of values for the
            ' iterator
        
            ' Write any last minute changes to the grid
            Call CopyTextToGrid(frmSteps.msItValues, _
                    frmSteps.txtEdit)
        
            ' Create iterator records of type Value for each
            ' entry in the Iterator Values grid
            intSequence = gintMinIteratorSequence
            For intIndex = 0 To frmSteps.msItValues.Rows - 1
                strValue = Trim$(frmSteps.msItValues.TextMatrix( _
                            intIndex, mintColValue))
                
                If Not StringEmpty(strValue) Then
                    Set cItRecord = New cIterator
                    cItRecord.IteratorType = gintValue
                    cItRecord.SequenceNo = intSequence
                    cItRecord.Value = strValue
                    
                    cItValues.Load cItRecord
                    intSequence = intSequence + 1
                End If
            Next intIndex
        End If
    End If
    
    Set GetItValuesFromScreen = cItValues
    
    Exit Function

GetItValuesFromScreenErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errReadFromScreenFailed, _
            gstrSource, LoadResString(errReadFromScreenFailed)

End Function
Public Sub ValidateItValues(cStepRec As cStep)
    ' Checks if the iterator values entered by the user
    ' on the Step Properties screen are valid

    Dim strOuterLoop As String
    Dim strInnerLoop As String
    Dim lngIndex As Long
    Dim lngComp As Long
    Dim iRange(mintFrom To mintStep) As Integer
    Dim sRange(mintFrom To mintStep) As String
    Dim sTemp As String
    
    On Error GoTo ValidateItValuesErr
    gstrSource = mstrModuleName & "ValidateItValues"

    If Not StringEmpty(frmSteps.txtIterator) Then
        ' Make sure that the iterator name is unique
        Call ValidateIteratorName(frmSteps.txtIterator.Text, cStepRec)
    
        If frmSteps.optItType(mintItTypeNumeric) Then
            ' The user has entered a range of values for the
            ' iterator - make sure that the values entered are
            ' numeric
            
            For lngIndex = frmSteps.txtItNumValues.LBound To _
                    frmSteps.txtItNumValues.UBound
                sRange(lngIndex) = Trim$(frmSteps.txtItNumValues(lngIndex))
                sTemp = SubstituteParameters(sRange(lngIndex), cStepRec.WorkspaceId)
                
                If Not IsNumeric(sTemp) Then
                    ShowError errRangeNumeric
                    On Error GoTo 0
                    Err.Raise vbObjectError + errRangeNumeric, _
                            gstrSource, _
                            LoadResString(errRangeNumeric)
                End If
                
                iRange(lngIndex) = CInt(sTemp)
            Next lngIndex
            
            ' A value of 0 for the step will put us in an
            ' infinite loop
            ' Also check if the the iterator can assume
            ' any values in the specified range
            If (iRange(mintStep) = 0) Or _
                    (iRange(mintFrom) > iRange(mintTo) And iRange(mintStep) > 0) Or _
                    (iRange(mintFrom) < iRange(mintTo) And iRange(mintStep) < 0) Then
                ShowError errRangeInvalid
                On Error GoTo 0
                Err.Raise vbObjectError + errRangeInvalid, _
                        gstrSource, _
                        LoadResString(errRangeInvalid)
            End If
        Else
            ' The user has entered a set of values for the
            ' iterator
            
            For lngIndex = 0 To frmSteps.msItValues.Rows - 1
                strOuterLoop = LCase$(Trim$( _
                        frmSteps.msItValues.TextMatrix( _
                        lngIndex, mintColValue)))
                
                If Not StringEmpty(strOuterLoop) Then
                    ' Check for duplicate values within the values
                    ' defined for the step
                    For lngComp = lngIndex + 1 To frmSteps.msItValues.Rows - 1
                        strInnerLoop = LCase$(Trim$( _
                                frmSteps.msItValues.TextMatrix( _
                                lngComp, mintColValue)))
                        If strInnerLoop = strOuterLoop Then
                            ShowError errDuplicateIterator
                            On Error GoTo 0
                            Err.Raise errDuplicateIterator, _
                                    gstrSource, _
                                    LoadResString(errDuplicateIterator)
                        End If
                    Next lngComp
                End If
            Next lngIndex
        End If
    End If
                            
    Exit Sub

ValidateItValuesErr:
    ' Log the error code raised by Visual Basic
    gstrSource = mstrModuleName & "ValidateItValues"
    If Err.Number = vbObjectError + errSubValuesFailed Then
        ShowError errRangeNumeric
    End If

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errValidateFailed, gstrSource, _
        LoadResString(errValidateFailed)

End Sub
Private Sub ModifyIteratorSeq(cStepRec As cStep, _
        cDeleteIt As cIterator)
    ' Decrements the sequence numbers of all the iterators
    ' that appear after the iterator being deleted
    
    Dim vntIterators As Variant
    Dim cItRec As cIterator
    Dim lngIndex As Long
    Dim lngSequence As Long
    
    On Error GoTo ModifyIteratorSeqErr
    gstrSource = mstrModuleName & "ModifyIteratorSeq"
    
    ' Call a function that will return all the iterators
    ' for the step
    vntIterators = cStepRec.Iterators()
    
    lngSequence = cDeleteIt.SequenceNo
    
    If Not IsEmpty(vntIterators) And IsArray(vntIterators) Then
        For lngIndex = LBound(vntIterators) To _
                UBound(vntIterators)
                
            Set cItRec = vntIterators(lngIndex)
            
            If cItRec.SequenceNo > lngSequence Then
                ' Decrement the sequence number for the
                ' iterator record by 1
                cItRec.SequenceNo = cItRec.SequenceNo - 1
                cStepRec.ModifyIterator cItRec
            End If
        
        Next lngIndex
    End If
        
    Exit Sub
    
ModifyIteratorSeqErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ModifyIteratorSeq"
    On Error GoTo 0
    Err.Raise vbObjectError + errUpdateSequenceFailed, _
            gstrSource, _
            LoadResString(errUpdateSequenceFailed)
    
End Sub
Public Sub WriteIterators(cStepRecord As cStep)

    ' Adds/updates/deletes iterator records for the step -
    ' these changes are not committed to the database yet!

    Dim cItRec As cIterator
    Dim cNewItRec As cIterator
    Dim lngIndex As Long
    Dim vntIterators As Variant
    Dim cNewIterators As cNodeCollections

    On Error GoTo WriteIteratorsErr

    ' Load a collection with the iterator values entered by
    ' the user
    Set cNewIterators = GetItValuesFromScreen

    ' Get all the iterators for the step
    vntIterators = cStepRecord.Iterators

    If Not IsEmpty(vntIterators) And IsArray(vntIterators) Then
        
        For lngIndex = LBound(vntIterators) To UBound(vntIterators)

            Set cItRec = vntIterators(lngIndex)

            Set cNewItRec = GetIterator(cNewIterators, _
                    cItRec.IteratorType, cItRec.Value)
                    
            If cItRec.IteratorType = gintValue Then
                If cNewItRec Is Nothing Then
                    ' The iterator does not exist in the grid,
                    ' meaning that it has been deleted by the user
                    cStepRecord.RemoveIterator cItRec
                Else
                    If cNewItRec.SequenceNo <> cItRec.SequenceNo Then
                        ' If the sequence of the iterator has
                        ' changed, update it
                        cItRec.SequenceNo = cNewItRec.SequenceNo
                        cStepRecord.ModifyIterator cItRec
                    End If
                    ' Remove the iterator record from the
                    ' collection - this ensures that once we're
                    ' out of this loop, all iterators will be
                    ' newly inserted ones
                    cNewIterators.Unload cNewItRec.Position
                End If
            Else
                If cNewItRec Is Nothing Then
                    ' The iterator does not exist in the grid,
                    ' meaning that it has been deleted by the user
                    cStepRecord.RemoveIterator cItRec
                Else
                    If cNewItRec.Value <> cItRec.Value Then
                        ' If the sequence of the iterator has
                        ' changed, update it
                        cItRec.Value = cNewItRec.Value
                        cStepRecord.ModifyIterator cItRec
                    End If
                    ' Remove the iterator record from the
                    ' collection - this ensures that once we're
                    ' out of this loop, all iterators will be
                    ' newly inserted ones
                    cNewIterators.Unload cNewItRec.Position
                End If
            End If
        Next lngIndex
    End If

    ' Call a procedure to insert new iterator records,
    ' if any were created by the user
    Call NewIterators(cNewIterators, cStepRecord)
    
    Exit Sub

WriteIteratorsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errUnableToWriteChanges, _
            mstrModuleName & "WriteIterators", LoadResString(errUnableToWriteChanges)

End Sub
'Public Sub CopyIterators(cStepFrom As cStep, cStepTo As cStep)
'    ' Creates iterator records for the StepTo class
'    ' that are identical to existing iterator records
'    ' for StepFrom
'
'    Dim cItToCreate As cIterator
'    Dim intIndex As Integer
'    Dim vntIterators As Variant
'
'    On Error GoTo CopyIteratorsErr
'
'    ' Call a function that will return all the iterators
'    ' that have been defined for the step From
'    vntIterators = cStepFrom.Iterators
'
'    If Not IsEmpty(vntIterators) And IsArray(vntIterators) Then
'        For intIndex = LBound(vntIterators) To UBound(vntIterators)
'
'            ' Initialize the new iterator record to the
'            ' existing iterator
'            Set cItToCreate = vntIterators(intIndex).Clone
'
'            ' Add the iterator record to the new step
'            cStepTo.AddIterator cItToCreate
'
'            Set cItToCreate = Nothing
'
'        Next intIndex
'    End If
'
'    Exit Sub
'
'CopyIteratorsErr:
'    ' Log the error code raised by Visual Basic
'    Call LogErrors(Errors)
'    gstrSource = mstrModuleName & "CopyIterators"
'    On Error GoTo 0
'    Err.Raise vbObjectError + errCopyFailed, gstrSource, _
'        LoadResString(errCopyFailed)
'
'End Sub
Public Sub DeleteStepIterators(ByVal cDeleteStep As cStep)
    ' Deletes all the iterator records for the passed
    ' in step
    
    Dim vntIterators As Variant
    Dim cItRec As cIterator
    Dim lngIndex As Long
    
    On Error GoTo DeleteStepIteratorsErr
    
    ' Call a function that will return all the iterators
    ' that have been defined for the step
    vntIterators = cDeleteStep.Iterators
    
    If Not IsEmpty(vntIterators) And IsArray(vntIterators) Then
        For lngIndex = LBound(vntIterators) To UBound(vntIterators)
            Set cItRec = vntIterators(lngIndex)
            
            ' Delete the iterator from the collection
            cDeleteStep.RemoveIterator cItRec
        Next lngIndex
    End If
        
    Exit Sub
    
DeleteStepIteratorsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "DeleteStepIterators"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteFailed, _
            gstrSource, _
            LoadResString(errDeleteFailed)
    
End Sub
Public Sub LoadIterators(cStepRec As cStep)
    ' Loads the iterators for the passed in step on the
    ' step properties form
        
    Dim intIndex As Integer
    Dim vntIterators As Variant
    Dim cItRec As cIterator
    
    On Error GoTo LoadIteratorsErr
    
    ' Call a function that will return all the iterators
    ' that have been defined for the step
    vntIterators = cStepRec.Iterators
    
    If Not IsEmpty(vntIterators) And IsArray(vntIterators) Then
        ' Enable the Numeric values frame or the List frame,
        ' depending on the iterator type
        If vntIterators(LBound( _
                vntIterators)).IteratorType = gintValue Then
            ' Enable the grid used to populate the list of
            ' iterator values
            Call InitCtlArray(frmSteps.txtItNumValues, mintMinIterator)
            frmSteps.optItType(mintItTypeValues).Value = True
        Else
            RangeComplete (vntIterators)
            
            frmSteps.optItType(mintItTypeNumeric).Value = True
        End If
        
        For intIndex = LBound(vntIterators) To _
                UBound(vntIterators)

            Set cItRec = vntIterators(intIndex)
            
            If cItRec.IteratorType = gintValue Then
                ' Add the value to the grid
                Call SetValue(frmSteps.msItValues, _
                        cItRec)
            Else
                ' Initialize the corresponding text box
                ' with the numeric range
                frmSteps.txtItNumValues(cItRec.IteratorType) = _
                        cItRec.Value
            End If
        Next intIndex
    End If

    Exit Sub

LoadIteratorsErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "LoadIterators"
    Err.Raise vbObjectError + errLoadDataFailed, gstrSource, _
            LoadResString(errLoadDataFailed)
    
End Sub
